class GeoModel:

    def __init__(self, backend, name, dolead_id, publisher_id, **kwargs):
        self._backend = backend
        self.name = name
        self.dolead_id = dolead_id
        self.publisher_id = publisher_id

        self.slug = kwargs.get('slug')
        self.geo_type = kwargs.get('geo_type')
        self.parent_id = kwargs.get('parent_id')
        self.canonical_name = kwargs.get('canonical_name')
        self.country_code = kwargs.get('country_code')
        self.iso_code = kwargs.get('iso_code')

        self.latitude = kwargs.get('latitude')
        self.longitude = kwargs.get('longitude')

    @property
    def bing_id(self):
        return self.publisher_id

    @property
    def adwords_id(self):
        return self.publisher_id

    @property
    def name_full(self):
        """Get full name including hierarchy"""
        return ', '.join(reversed([p.name
                for p in self._backend.list_ancestors(self)]))

    @property
    def parent(self):
        return self._backend.get({'dolead_id': self.parent_id})

    def __str__(self):
        return "<%s #%s [%s] %s>" % (self.geo_type, self.dolead_id,
                                     self.country_code, self.name)

    def __repr__(self):
        return str(self)

    def __iter__(self):
        for child in self._backend.list_children(self):
            yield child

    def __hash__(self):
        return hash(self.dolead_id)

    def __getitem__(self, key, default=None):
        return getattr(self, key, default)

    def get(self, key, default=None):
        return getattr(self, key, default)

    def dump(self):
        return {'name': self.name,
                'dolead_id ': self.dolead_id,
                'publisher_id': self.publisher_id,

                'slug': self.slug,
                'geo_type': self.geo_type,
                'parent_id': self.parent_id,
                'canonical_name': self.canonical_name,
                'country_code': self.country_code,
                'iso_code': self.iso_code,

                'latitude': self.latitude,
                'longitude': self.longitude}



import logging
from ad_geo_backend.backends.mongodb import MongoBackend
from ad_geo_backend.utils import common, bing, google, cmd_line_utils

logger = logging.getLogger(__name__)


def load():
    google_file, bing_file, french_pc_file, iso_code_file \
            = cmd_line_utils.parse_args()
    google_backend = MongoBackend('GOOGLE')
    bing_backend = MongoBackend('BING') if bing_file else None

    slug_to_gps = None
    if french_pc_file:
        slug_to_gps = common.parse_french_pc(french_pc_file)

    iso_code_mapping = None
    if iso_code_file:
        iso_code_mapping = common.parse_iso_codes(iso_code_file)
    google_trans = google.Translator(slug_to_gps, iso_code_mapping)
    bing_trans = bing.Translator(slug_to_gps, iso_code_mapping) \
            if bing_file else None

    if google_file:
        google_backend.reset()
        google_backend.check_indexes()
        common.load_file_to_backend(google_backend, google_file, google_trans)
    if bing_file:
        bing_backend = MongoBackend('BING')
        bing_backend.reset()
        bing_backend.check_indexes()
        common.load_file_to_backend(bing_backend, bing_file, bing_trans)
        bing.correct_with_google_data(bing_backend, google_backend)



#!/usr/bin/env python
from setuptools import setup, find_packages

REQUIREMENTS = ['pymongo>=2.8', 'python-slugify==1.2.6']


setup(name='ad-geo-backend',
      version='1.1.2',
      description='Geographical data management',
      url='https://github.com/dolead/ad-geo-backend',
      author_email='backend@dolead.com',
      install_requires=REQUIREMENTS,
      packages=find_packages(),
      entry_points={
          'console_scripts': [
              'load-geo-data = ad_geo_backend.shell:load',
          ],
      },
)



import re
import logging
from slugify import slugify
from ad_geo_backend.backends.mongodb import MongoBackend
from ad_geo_backend.utils import file_utils

logger = logging.getLogger(__name__)
UPDATE_FREQ = 500
WRITE_BATCH_SIZE = 10000
SAINT_PATTERNS = ((re.compile(r'^st-'), 'saint-'),
                  (re.compile(r'-st-'), '-saint-'),
                  (re.compile(r'^ste-'), 'sainte-'),
                  (re.compile(r'-ste-'), '-sainte-'))


class AbstractTranslator:

    def __init__(self, slug_to_gps=None, gid_to_iso=None):
        self._stats = {'translated': 0, 'gps_coord_added': 0,
                       'iso_code_added': 0}
        self._slug_to_gps = slug_to_gps or {}
        self._gid_to_iso = gid_to_iso or {}

    def _enrich_w_iso_code(self, google_id, result):
        if google_id in self._gid_to_iso:
            result['iso_code'] = self._gid_to_iso[google_id]
            self._stats['iso_code_added'] += 1

    def _enrich_w_coords(self, slug, result):
        if slug in self._slug_to_gps \
                and result['canonical_name'].endswith('France'):
            result['latitude'] = self._slug_to_gps[slug][0]
            result['longitude'] = self._slug_to_gps[slug][1]
            self._stats['gps_coord_added'] += 1

    def print_stats(self):
        print('translated ', self._stats['translated'], ' elements')
        print('added gps coords to ', self._stats['gps_coord_added'],
                ' elements')
        print('added iso code to ', self._stats['iso_code_added'], ' elements')


def parse_french_pc(french_pc_file):
    slug_to_gps = {}
    for city in file_utils.csv_to_dict(french_pc_file, delimiter=';'):
        if not city.get('coordonnees_gps'):
            continue
        gps_coords = tuple(city['coordonnees_gps'].split(', '))
        for key in ['Nom_commune', 'Libelle_acheminement',
                    'Ligne_5', 'Code_postal']:
            if not city.get(key):
                continue
            key = slugify(city[key])
            slug_to_gps[key] = gps_coords
            for pattern, long_form in SAINT_PATTERNS:
                if pattern.match(key):
                    slug_to_gps[pattern.sub(long_form, key)] = gps_coords
    return slug_to_gps


def parse_iso_codes(iso_codes_csv):
    iso_mapping = {}
    for iso_code in file_utils.csv_to_dict(iso_codes_csv):
        iso_mapping[iso_code['Google ID']] = iso_code['Iso Code']
    return iso_mapping


def correct_with_french_geoloc(network, cities, postal_codes):
    backend = MongoBackend(network)
    modified_cities, modified_pc = 0, 0

    logger.warning('adding lat/long to french cities')
    for index, slug in enumerate(cities):
        if not cities[slug]['coordonnees_gps']:
            continue
        if not index % UPDATE_FREQ:
            logger.warning('FRENCH CITIES modified/processed/total %d/%d/%d',
                    modified_cities, index, len(cities))
        lat, lng = cities[slug]['coordonnees_gps'].split(', ')
        result = backend.collection.update(
                {'country_code': 'FR', 'slug': slug},
                {'$set': {'latitude': lat, 'longitude': lng}})
        modified_cities += result['nModified']

    logger.warning('adding lat/long to french postal codes')
    for index, postal_code in enumerate(postal_codes):
        if not postal_codes[postal_code]['coordonnees_gps']:
            continue
        if not index % UPDATE_FREQ:
            logger.warning('FRENCH PC modified/processed/total %d/%d/%d',
                    modified_pc, index, len(cities))
        lat, lng = postal_codes[postal_code]['coordonnees_gps'].split(', ')
        result = backend.collection.update(
                {'country_code': 'FR', 'name': postal_code},
                {'$set': {'latitude': lat, 'longitude': lng}})
        modified_pc += result['nModified']

    logger.warning('Updated %d objects', modified_cities + modified_pc)


def load_file_to_backend(backend, csv_file, translator):
    lines = []
    for line in file_utils.csv_to_dict(csv_file):
        line = translator.translate(line)
        lines.append(line)
        if len(lines) >= WRITE_BATCH_SIZE:
            logger.info('writing %d lines into %s',
                    WRITE_BATCH_SIZE, backend.network)
            backend.insert_many(lines)
            lines = []
    if lines:
        backend.insert_many(lines)
        logger.info('writing %d lines into %s', len(lines), backend.network)
    translator.print_stats()



import logging
from slugify import slugify
from ad_geo_backend.utils.common import AbstractTranslator

logger = logging.getLogger(__name__)


class Translator(AbstractTranslator):

    def __init__(self, *args, **kwargs):
        self.parents = {}
        super().__init__(*args, **kwargs)

    def translate(self, line):
        names = line['Bing Display Name'].split('|')
        dolead_id = line['AdWords Location Id'] \
                or 'bing_%s' % line['Location Id']
        self.parents[tuple(names)] = dolead_id

        result = {'dolead_id': dolead_id,
                  'publisher_id': line['Location Id'],
                  'geo_type': line['Location Type'],
                  'name': names[0],
                  'slug': slugify(names[0]),
                  'canonical_name': line['Bing Display Name'],
                  'google_ref': bool(line.get('AdWords Location Id')),
                  }

        if len(names) > 1:
            result['parent_id'] = self.parents[tuple(names[1:])]
            result['top_level'] = names[-1]

        self._enrich_w_coords(result['slug'], result)
        self._enrich_w_iso_code(dolead_id, result)
        self._stats['translated'] += 1
        return result


def correct_with_google_data(bing_backend, google_backend):
    logger.warning('adding country code to bing datas')
    country_codes = {country.name: country.country_code
            for country in google_backend.list({'geo_type': 'Country'})}

    modified_elements = 0
    for index, name in enumerate(country_codes):
        if not index % 50:
            logger.warning('added %d/%d codes (to %d locations)',
                    index, len(country_codes), modified_elements)
        res1 = bing_backend.collection.update({'top_level': name},
                {'$set': {'country_code': country_codes[name]}}, multi=True)
        res2 = bing_backend.collection.update(
                {'name': name, 'geo_type': 'Country'},
                {'$set': {'country_code': country_codes[name]}}, multi=True)
        modified_elements += res1['nModified'] + res2['nModified']
    logger.warning('added %d/%d codes (to %d locations)',
            len(country_codes), len(country_codes), modified_elements)



from slugify import slugify
from ad_geo_backend.utils.common import AbstractTranslator


class Translator(AbstractTranslator):

    def translate(self, line):
        result = {'dolead_id': line['Criteria ID'],
                  'publisher_id': line['Criteria ID'],
                  'geo_type': line['Target Type'],
                  'country_code': line['Country Code'],
                  'name': line['Name'],
                  'slug': slugify(line['Name']),
                  'canonical_name': line['Canonical Name'],
                  }
        if line.get('Parent ID'):
            result['parent_id'] = line['Parent ID']
        self._enrich_w_coords(result['slug'], result)
        self._enrich_w_iso_code(line['Criteria ID'], result)
        self._stats['translated'] += 1
        return result



# -*- coding: utf8 -*-
import logging

from pymongo import MongoClient
from pymongo.database import Database

from ad_geo_backend.backends.abstract import AbstractGeoBackend
from ad_geo_backend.model import GeoModel

logger = logging.getLogger(__name__)


class MongoBackend(AbstractGeoBackend):
    __connection = None
    __db_name = None
    __collection_prefix = 'ad_geo_backend'
    __indexes = {
        'dolead_id': {'unique': True},
        'name': None,
        'slug': None,
        'canonical_name': None,
        'country_code': None,
        'parent_id': None,
        'publisher_id': None,
    }

    @classmethod
    def set_connection(cls, host, db_name, **credentials):
        if cls.__connection is None:
            cls.__connection = MongoClient(host)
            cls.__db_name = db_name
            if len({'name', 'password'}.intersection(credentials)) == 2:
                cls.get_database().authenticate(**credentials)

    def list(self, criteria, sort=None, limit=None):
        query = self.collection.find(criteria)
        if sort:
            query.sort(sort)
        if limit:
            query.limit(limit)
        for l in query:
            yield GeoModel(self, **l)

    def get(self, criteria):
        item = self.collection.find_one(criteria)
        return GeoModel(self, **item) if item else item

    def insert_many(self, objects):
        return self.collection.insert(objects)

    def reset(self):
        return self.collection.drop()

    # Mongo specific methods

    @classmethod
    def get_database(cls):
        if not cls.__connection:
            raise RuntimeError('Please set an MongoDB connection first')
        return Database(cls.__connection, cls.__db_name)

    @property
    def collection(self):
        return getattr(self.get_database(),
                '%s_%s' % (self.__collection_prefix, self.network.lower()))

    def check_indexes(self):
        for idx, options in self.__indexes.items():
            options = options or {}
            self.collection.ensure_index(idx, **options)
